/*
 * cmsfs-fuse - CMS EDF filesystem support for Linux
 * Data structures.
 *
 * Copyright IBM Corp. 2009
 * Author(s): Jan Glauber <jang@linux.vnet.ibm.com>
 */

#ifndef _CMSFS_H
#define _CMSFS_H

#define COMP "cmsfs-fuse: "

extern FILE *logfile;

#ifdef DEBUG_ENABLED
#define DEBUG(...)					\
	do {						\
		fprintf(logfile, __VA_ARGS__);		\
		fflush(logfile);			\
	} while (0)
#else
#define DEBUG(...)
#endif

#define DIE(...)					\
	do {						\
		fprintf(stderr, COMP __VA_ARGS__);	\
		exit(1);				\
	} while (0)

#define DIE_PERROR(...)					\
	do {						\
		perror(COMP __VA_ARGS__);		\
		exit(1);				\
	} while (0)

#define BUG(x)								\
	if (x) {							\
		fprintf(stderr, COMP " assert failed at "		\
			__FILE__ ":%d in %s()\n", __LINE__, __func__);	\
		exit(1);						\
	}

#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof((arr)[0]))

struct fcache_entry {
	/* filename used as hash key */
	char		name[18];
	/* location of fst entry */
	off_t		fst_addr;
};

/* the per device global struture */
struct cmsfs {
	/* name of the block device, e.g. /dev/dasde */
	char		*device;
	/* global file descriptor of the underlying block device */
	int		fd;
	/* mmap of block device */
	char		*map;
	/* size of the disk in bytes */
	size_t		size;
	/* formatted blocksize */
	unsigned int	blksize;

	/* disk info */
	unsigned int	format;

	/* offset to file directory */
	unsigned int	fdir;
	/* offset to allocation map  */
	unsigned int	amap;
	/* depth of directory levels */
	int		levels;
	/* directory extension */
	int		fop;
	/* files count on the device */
	int		files;
	/* disable automatic translation to ascii */
	int		raw_mode;
	/* force translation to ascii */
	int		ascii_mode;
	/* translation to ascii based on file type */
	int		filetype_mode;

	/* disk stats */
	unsigned int	total_blocks;
	unsigned int	used_blocks;

	int		fixed_ptrs_per_block;
	int		var_ptrs_per_block;
	int		bits_per_data_block;
	int		bits_per_ptr_block;
	int		data_block_mask;

	/* file cache */
	struct		fcache_entry *fcache;
	int		fcache_used;
	void		*htab;
};

/* file status table entry */
struct fst_entry {
	char		name[8];
	char		type[8];
	char		res1[8];

	short int	mode;
	char		res2[4];

	char		record_format;
	char		flag;
	unsigned int	record_len;
	char		res3[4];

	int		fop;
	/* number of data blocks (not incl. pointer blocks and null blocks!) */
	int		nr_blocks;
	int		nr_records;
	char		levels;
	char		ptr_size;
	char		date[6];
	char		res4[4];
};

struct cms_label {
	char		id[6];
	char		user_id[6];
	unsigned int	blocksize;
	unsigned int	dop;
	unsigned int	f_cylinders;
	unsigned int	max_cylinders;
	unsigned int	total_blocks;
	unsigned int	used_blocks;
	unsigned int	fst_entry_size;
	unsigned int	fst_per_block;
	unsigned int	res1;
	unsigned int	res2;
	char		res3[8];
};

#define RECORD_LEN_VARIABLE	0xe5
#define RECORD_LEN_FIXED	0xc6

#define FST_ENTRY_SIZE		sizeof(struct fst_entry)
#define FST_ENTRY_DIR_NAME	0x0000000100000000ULL
#define FST_ENTRY_DIR_TYPE	0xc4c9d9c5c3e3d6d9ULL	/* 'DIRECTOR' */
#define FST_ENTRY_ALLOC_NAME	0x0000000200000000ULL
#define FST_ENTRY_ALLOC_TYPE	0xc1d3d3d6c3d4c1d7ULL	/* 'ALLOCMAP' */

#define FST_FLAG_CENTURY	0x0008
#define FST_FOP_OFFSET		0x28

#define VAR_RECORD_HEADER_SIZE	0x2
#define VAR_RECORD_SPANNED	0xffffffff

struct fixed_ptr {
	unsigned int next;
};

struct var_ptr {
	unsigned int next;
	int max_record_nr;
	unsigned int displacement;
};

extern int get_device_info(struct cmsfs *cmsfs);

#endif
